'use strict';

var dExt = window.dExt = (function() {


  return {
    version: vAPI.getManifest().version,
    onBeforeStartQueue: [],
    failedUploadsQueue: []
  };

})();

var stripMenuItemId = function(itemId) {
  var outId = itemId;
  if (outId.indexOf("addpin-") === 0) {
    outId = outId.substr(7);
  }
  if (outId.lastIndexOf("-page") === outId.length - 5 || outId.lastIndexOf("-link") === outId.length - 5) {
    outId = outId.substr(0, outId.length - 5)
  }
  return outId;
};

dExt.onContextMenuClicked = function(info, tab) {
  var self = this;
  if (info.menuItemId.indexOf("save-") === 0) {
    vAPI.cloud.get({
      saveformat: "png"
    }, function(config) {
      var formatConfig = config.saveformat;
      var item = {};
      if (info.menuItemId.indexOf('link') !== -1) {
        item.url = info.linkUrl || info.srcUrl || info.pageUrl;
        item.type = "link";
      }else {
        item.url = info.srcUrl || info.linkUrl || info.pageUrl;
        item.type = info.mediaType || (!!info.pageUrl? (info.menuItemId === "save-audio-forced" ? 'audio':'webpage') : '');
      }
      if (item.type === "webpage") {
        item.exportType = formatConfig;
      }

      if (item.url) {
        // Context menu on a file
        item.name = decodeURIComponent(item.url.split("/").pop()) || 'webcontent';
        vAPI.tabs.sendMessage(tab.id, {
          action: 'addFolderPicker',
          item:   item
        }, function(response) {});
      }
    });
  } else if (info.menuItemId.indexOf("addpin-") === 0) {
    var url;
    if (info.menuItemId.lastIndexOf("-page") === info.menuItemId.length - 5) {
      url = info.pageUrl || info.frameUrl;
    } else {
      url = info.linkUrl || info.srcUrl || info.pageUrl;
    }
    if (url) {
      vAPI.tabs.sendMessage(tab.id, {
        action: 'addAddPin',
        menuItemId: stripMenuItemId(info.menuItemId),
        url:   url
      }, function(response) {});
    }
  }
}

dExt.openOrFocusDriveTab = function(product) {
  var destUrl = dConfig.DESKTOP_URL;
  if (product) {
    switch (product) {
      case "library":
        destUrl = dConfig.LIBRARY_URL;
        break;
    }
  }
  vAPI.tabs.query({ url: destUrl+'/*' }, function(windowTabs) {
    if (windowTabs && windowTabs.length) {
      vAPI.tabs.highlight({ windowId: windowTabs[0].windowId, tabs: windowTabs[0].index }, function(browserWindow){
        vAPI.windows.update(browserWindow.id, { focused: true }, function (){});
      });
    } else {
      vAPI.tabs.create({ url: destUrl});
    }
  });
};

dExt.afterUploadHandler = function(err, uploadRequest) {
  var file = uploadRequest.file;
  var type = uploadRequest.type;
  var shortName = file.name.length <= 20? file.name : file.name.substr(0,20)+"...";

  if (err) {
    var message = "Error: "+shortName+" was not saved.";
    if (type) {
      switch (type) {
        case 'torrent':
          message = "Error: the download of "+shortName+" could not be started.";
          break;
      }
    }
    //there was an error during upload
    var notifId = vAPI.notifications.push("desktopExtensionUploadFailed",{
      type: "basic",
      iconUrl: "assets/images/desktop-logo-64-failed.png",
      title: "Failed to Save to your Desktop",
      message: message,
      isClickable: true,
      buttons: [{
        title: "Retry",
      }]
    });
    this.failedUploadsQueue.push({
      id: notifId,
      uploadRequest: uploadRequest
    });
    console.error(err);
  } else {
    var message = "the file "+shortName+" was saved successfully!";
    if (type) {
      switch (type) {
        case 'audio':
          message = "the track "+shortName+" was saved successfully!";
          break;
        case 'video':
          message = "the video "+shortName+" was saved successfully!";
          break;
        case 'image':
          message = "the image "+shortName+" was saved successfully!";
          break;
        case 'webpage':
          message = "the page "+shortName+" was saved successfully!";
          break;
        case 'torrent':
          message = "the download of "+shortName+" started successfully!";
          break;
      }
    }
    vAPI.notifications.push("desktopExtensionUploadComplete",{
      type: "basic",
      iconUrl: "assets/images/desktop-logo-64-complete.png",
      title: "Saved to your Desktop",
      message: message,
      isClickable: true
    });
  }
}

dExt.afterAddPinHandler = function(err, addPinRequest) {
  var shortName = addPinRequest.nodeName.length <= 20? addPinRequest.nodeName : addPinRequest.nodeName.substr(0,20)+"...";

  if (err) {
    var message = "Error: "+shortName+" was not added.";
    //there was an error during upload
    var notifId = vAPI.notifications.push("desktopExtensionAddPinFailed",{
      type: "basic",
      iconUrl: "assets/images/desktop-logo-64-failed.png",
      title: "Failed to Add the channel",
      message: message,
      isClickable: true
    });
    console.error(err);
  } else {
    var message = "the channel "+shortName+" was added successfully!";
    vAPI.notifications.push("desktopExtensionAddPinComplete",{
      type: "basic",
      iconUrl: "assets/images/desktop-logo-64-complete.png",
      title: "Channel added to your Library",
      message: message,
      isClickable: true
    });
  }
}

dExt.uploader = function(uploadRequest, sender) {
  var self = this;
  var file = uploadRequest.file;
  var destinationConnection = uploadRequest.destinationConnection;
  var type = uploadRequest.type;
  vAPI.net.uploadGql(destinationConnection, file, type, function(err) {
    self.afterUploadHandler(err, uploadRequest);
  });
  if (sender) {
    vAPI.tabs.sendMessage(sender.tab.id, { 'action': 'remove' }, function(response) {});
  }
}

dExt.pinAdder = function(addPinRequest, sender) {
  var self = this;
  var pin = {
    "serviceId": addPinRequest.service,
    "nodeId": addPinRequest.nodeId,
    "nodeName": addPinRequest.nodeName,
    "query": addPinRequest.query,
    "thumbnail": addPinRequest.thumbnail,
    "url": addPinRequest.url,
  };
  vAPI.net.addPin(pin, function(err) {
    self.afterAddPinHandler(err, addPinRequest);
  });
  if (sender) {
    vAPI.tabs.sendMessage(sender.tab.id, { 'action': 'remove' }, function(response) {});
  }

}

dExt.initMessaging = function() {
  var self = this;
  // Listen to event from external webpages
  vAPI.onMessageExternal.addListener(function(request, sender, sendResponse) {
    if (request.action) {// pipe to the page script
      vAPI.tabs.sendMessage(sender.tab.id, {
        action:   request.action,
        external: request
      }, function(response) {});
    } else if (request.upload) {// background upload request
      self.uploader(request.upload, sender);
    } else if (request.addpin) {// background upload request
      self.pinAdder(request.addpin, sender);
    } else if (request.goto) {// request to go to a product tab
      var product = request.goto.product.toLowerCase();
      self.openOrFocusDriveTab(product);
    } else if (request.message) {
      if (request.message == "version") {
        sendResponse({version: self.version});
      }
    }

  });
    vAPI.onMessage.addListener(function(request, sender, sendResponse) {
      if (request.upload) {// background upload request
        if (dConfig.BROWSER === "firefox") {
          self.uploader(request.upload, sender);
        }
      } else if (request.addpin) {// background upload request
          self.pinAdder(request.addpin, sender);
      } else if (request.feed) {
        vAPI.tabs.executeScript(sender.tab.id,
            { code: "if (history.length > 1) " +
                     "history.go(-1); else window.close();"
            });
        var url = "rss_feed/subscribe.html?" + encodeURIComponent(request.feed);
        url = vAPI.getURL(url);
        vAPI.tabs.create({ url: url, index: sender.tab.index + 1 });
      } else if (request.addpinfeed) {
        vAPI.tabs.sendMessage(sender.tab.id, {
          action: 'addAddPin',
          menuItemId: "rss-feed-channel",
          url:   request.addpinfeed
        }, function(response) {});
      }
    });
}

dExt.initNotifActions = function() {
  var self = this;
  vAPI.notifications.addListener(function(notifId) {
    if (notifId.indexOf("desktopExtensionUploadFailed") === 0) {
      var uploadId = parseInt(notifId.substr(28));
      var failedUploadIdx = self.failedUploadsQueue.findIndex(function(upd) {
        return upd.id === uploadId;
      });
      if (failedUploadIdx !== -1) {
        self.uploader(self.failedUploadsQueue[failedUploadIdx].uploadRequest);
        self.failedUploadsQueue.splice(failedUploadIdx,1);
      }
    }
  }, {index: 0});
  // clicking on the body of the notif takes you to the product
  vAPI.notifications.addListener(function(notifId) {
    if (notifId.indexOf("desktopExtensionUploadComplete") === 0 || notifId.indexOf("desktopExtensionUploadFailed") === 0) {
      self.openOrFocusDriveTab();
    }
  });
}
