'use strict';

var dPatternStore = window.dPatternStore = (function() {
  var staticContextMenus = [
    {
      id: 'save-audio-forced',
      title: 'Save audio to your Desktop',
      contexts: ["page"],
      documentUrlPatterns: ["*://*/*.mp3","*://*/*.wav","*://*/*.ogg","*://*/*.flac","*://*/*.aac","*://*/*.MP3","*://*/*.WAV"]
    },
    {
      id: 'save-page-pdf',
      title: 'Save page to your Desktop',
      contexts: ["page"]
    },
    {
      id: 'save-image',
      title: 'Save image to your Desktop',
      contexts: ["image"],
    },
    {
      id: 'save-audio',
      title: 'Save audio to your Desktop',
      contexts: ["audio"],
    },
    {
      id: 'save-video',
      title: 'Save video to your Desktop',
      contexts: ["video"],
    },
  ];
  if (dConfig.BROWSER === "chrome") {
    staticContextMenus.push({
      id: 'save-link-magnet',
      title: 'Download to your Desktop',
      contexts: ["link"],
      targetUrlPatterns: ["magnet:*"]
    });
    staticContextMenus.push({
      id: 'save-link',
      title: "Save link's target to your Desktop",
      contexts: ["link"],
      targetUrlPatterns: ["http://*/*", "https://*/*", "ftp://*/*", "file://*/*" ]
    });
  } else if (dConfig.BROWSER === "firefox") {
    staticContextMenus.push({
      id: 'save-link',
      title: "Save link's target to your Desktop",
      contexts: ["link"]
    });
  }
    return {
      settings: {
        hasInit: false,
        contextMenus: null,
        autoUpdate: true,
        autoUpdatePatternsPeriod: 24 * 60 * 60 * 1000, // time between pattern fetches
        startupUpdateWaitingPeriod: 10 * 1000,
        initialUpdateWaitingPeriod: 30 * 1000
      },
      staticContextMenus: staticContextMenus,
      storageExceptions: ["autoUpdate", "autoUpdatePatternsPeriod", "startupUpdateWaitingPeriod", "initialUpdateWaitingPeriod"]
    };
})();

dPatternStore.updateSettings = function(updates, callback) {
  if (!updates) {
    return;
  }
  const updateKeys = Object.keys(updates);
  let hasUpdates = false;
  updateKeys.forEach(key => {
    this.settings[key] = updates[key];
    hasUpdates = true;
  });
  if (hasUpdates) {
    this.saveSettings(callback);
  } else {
    callback && callback();
  }
}

dPatternStore.saveSettings = function(callback) {
    vAPI.storage.set(this.settings, callback);
};

/******************************************************************************/

dPatternStore.updateStart = function(force, isInit) {
  //TODO fetch
  var self = this;
  vAPI.net.getContextMenus(function(err, newContextMenus) {
    if (err) {
      console.error(err);
      self.updateContextMenus(false, isInit);
      self.schedulePatternUpdater(self.settings.autoUpdatePatternsPeriod);
      return;
    }
    self.updateSettings({
      hasInit: true,
      contextMenus: newContextMenus
    }, function() {
      self.updateContextMenus(force);
      self.schedulePatternUpdater(self.settings.autoUpdatePatternsPeriod);
    });
  });
}

dPatternStore.updateContextMenus = function(force, isInitLoop) {
  if (!this.settings.contextMenus && !isInitLoop) {
    if (!this.settings.hasInit) {
      this.updateStart(force, true);
    }
    return;
  }
  vAPI.contextMenu.setEntries(this.decompressContextMenus(), dExt.onContextMenuClicked, force)
}

dPatternStore.decompressContextMenus = function() {
  let outputMenus = (this.staticContextMenus && this.staticContextMenus.slice()) || [];
  if (this.settings.contextMenus && this.settings.contextMenus.addPin) {
    this.settings.contextMenus.addPin.forEach(function(menuItem) {
      outputMenus.push({
        id: "addpin-"+menuItem.id+"-page",
        title: 'Add '+(menuItem.name || menuItem.type)+' to your Library',
        contexts: ["page"],
        documentUrlPatterns: menuItem.patterns
      });
      outputMenus.push({
        id: "addpin-"+menuItem.id+"-link",
        title: 'Add '+(menuItem.name || menuItem.type)+' to your Library',
        contexts: ["link"],
        targetUrlPatterns: menuItem.patterns
      });
    })
  }
  return outputMenus;
}

/******************************************************************************/

dPatternStore.schedulePatternUpdater = (function() {
    var timer, next = 0;
    return function(updateDelay) {
      var self = this;
      if ( timer ) {
          clearTimeout(timer);
          timer = undefined;
      }
      if ( updateDelay === 0 ) {
          next = 0;
          return;
      }
      var now = Date.now();
      // Use the new schedule if and only if it is earlier than the previous
      // one.
      if ( next !== 0 ) {
          updateDelay = Math.min(updateDelay, Math.max(next - now, 0));
      }
      next = now + updateDelay;
      timer = vAPI.setTimeout(function() {
          timer = undefined;
          next = 0;
          self.updateStart();
      }, updateDelay);
    };
})();
