'use strict';

dExt.restart = (function() {
  var processCallbackQueue = function(queue, callback) {
      var processOne = function() {
          var fn = queue.pop();
          if ( fn ) {
              fn(processOne);
          } else if ( typeof callback === 'function' ) {
              callback();
          }
      };
      processOne();
  };

  /******************************************************************************/

  var toFetch = function(from, fetched) {
      for ( var k in from ) {
          if ( from.hasOwnProperty(k) === false ) {
              continue;
          }
          fetched[k] = from[k];
      }
  };

  var fromFetch = function(to, fetched, exceptions) {
      for ( var k in to ) {
          if ( to.hasOwnProperty(k) === false ) {
              continue;
          }
          if ( fetched.hasOwnProperty(k) === false ) {
              continue;
          }
          if (exceptions.some(key => key === k)) {
              continue;
          }
          to[k] = fetched[k];
      }
  };

  /******************************************************************************/

  var onAllReady = function() {
      // https://github.com/chrisaljoudi/uBlock/issues/184
      // Check for updates not too far in the future.
      var waitingPeriod = dPatternStore.settings.startupUpdateWaitingPeriod;
      if (!dPatternStore.settings.hasInit) {
        waitingPeriod = dPatternStore.settings.initialUpdateWaitingPeriod;
      }
      dPatternStore.schedulePatternUpdater(dPatternStore.settings.autoUpdate ? waitingPeriod : 0);
      dPatternStore.updateContextMenus();
      dExt.initMessaging();
      dExt.initNotifActions();
  };

  var onFirstFetchReady = function(fetched) {
      fromFetch(dPatternStore.settings, fetched, dPatternStore.storageExceptions);
      onAllReady();

  };

  var loadFetchableProps = function() {
    var fetchableProps = {
    };
    toFetch(dPatternStore.settings, fetchableProps);
    vAPI.storage.get(fetchableProps, onFirstFetchReady);
  }

  /******************************************************************************/

  return function() {
    processCallbackQueue(dExt.onBeforeStartQueue, function() {
        loadFetchableProps();
    });
  };
})();

dExt.restart();
