'use strict';


var vAPI = window.vAPI;

(function(self) {
  var chrome = self.chrome;
  vAPI.tabs = chrome.tabs;
  vAPI.windows = chrome.windows;
  vAPI.browserAction = chrome.browserAction;
  vAPI.onMessageExternal = chrome.runtime.onMessageExternal;
  vAPI.contextMenu = chrome.contextMenus && {
      _callback: null,
      _entries: [],
      _createEntry: function(entry) {
          chrome.contextMenus.create(JSON.parse(JSON.stringify(entry)), function() {
            chrome.runtime.lastError && console.error(chrome.runtime.lastError.message);
            void chrome.runtime.lastError;
          });
      },
      onMustUpdate: function() {},
      setEntries: function(entries, callback, forceUpdate) {
          entries = entries || [];
          var n = Math.max(this._entries.length, entries.length),
              oldEntryId, newEntry;
          for ( var i = 0; i < n; i++ ) {
              oldEntryId = this._entries[i];
              newEntry = entries[i];
              if ( oldEntryId && newEntry ) {
                  if ( newEntry.id !== oldEntryId  || forceUpdate) {
                      chrome.contextMenus.remove(oldEntryId, function() {
                        chrome.runtime.lastError && console.error(chrome.runtime.lastError.message);
                        void chrome.runtime.lastError;
                      });
                      this._createEntry(newEntry);
                      this._entries[i] = newEntry.id;
                  }
              } else if ( oldEntryId && !newEntry ) {
                  chrome.contextMenus.remove(oldEntryId, function() {
                    chrome.runtime.lastError && console.error(chrome.runtime.lastError.message);
                    void chrome.runtime.lastError;
                  });
              } else if ( !oldEntryId && newEntry ) {
                  this._createEntry(newEntry);
                  this._entries[i] = newEntry.id;
              }
          }
          n = this._entries.length = entries.length;
          callback = callback || null;
          if ( callback === this._callback ) {
              return;
          }
          if ( n !== 0 && callback !== null ) {
              chrome.contextMenus.onClicked.addListener(callback);
              this._callback = callback;
          } else if ( n === 0 && this._callback !== null ) {
              chrome.contextMenus.onClicked.removeListener(this._callback);
              this._callback = null;
          }
      }
  };

  vAPI.net = {};
  //GraphQL query generator to upload the file
  var createUploadQuery = function(connectionId, folderId, url, name, mime, convertFile) {
    return JSON.stringify({
      "query": "mutation CreateLeaf($connectionId: ID!, $parentNodeId: ID!, $name: String!, $mime: String!, $url: String, $size: String, $useMultipartFile: Boolean, $convertFile: String, $isFrom: String) {\
        createLeaf(connectionId: $connectionId, parentNodeId: $parentNodeId, name: $name, mime: $mime, fromUrl: $url, size: $size, useMultipartFile: $useMultipartFile, convertFile: $convertFile, isFrom: $isFrom) {\
          ... on NodeAndUser {\
            node {\
              id\
            }\
            __typename\
          }\
          ... on Ongoing {\
            ongoing\
            __typename\
          }\
          __typename\
        }\
      }",
      "variables": {
        "connectionId": connectionId,
        "parentNodeId": folderId || "root",
        "name": name || "untitled",
        "mime": mime,
        "url": url,
        "useMultipartFile":false,
        "size":null,
        "convertFile": convertFile,
        "isFrom": "extension"
      },
      "operationName":"CreateLeaf"
    });
  };

  vAPI.net.uploadGql = function(destinationConnection, file, type, callback) {
    var url = dConfig.API_URL+'/graphql';
    var requestBody = createUploadQuery(destinationConnection, file.parentId, file.url, file.name, file.mime, file.convertFile || null);
    var myHeaders = new Headers({
      "Content-Type": "application/json",
    });
    fetch(url, {
      method: "POST",
      credentials: "include",
      headers: myHeaders,
      body: requestBody
    })
    .then(raw => {
      if (raw.status >= 400) {
        return Promise.reject("error sending the file");
      } else {
        return raw.json();
      }
    })
    .then(res => {
      var data = res.data;
      if (data && data.createLeaf && ((data.createLeaf.node && data.createLeaf.node.id) || data.createLeaf.ongoing)) {
        // upload went well
        return;
      } else {
        return Promise.reject("error saving the file");
      }
    })
    .then(function() {
      callback && callback();
    })
    .catch(function(err) {
      callback && callback(err);
    })
  };

  var getVersions = function() {
    var manifestData = chrome.runtime.getManifest();
    var extVersion = manifestData && manifestData.version;
    var sUsrAg = navigator.userAgent;
    var browserType = sUsrAg.indexOf("Chrome") > -1? "c":"f";
    var browserVersion;
    if (browserType === "c") {
      browserVersion = (/Chrome\/([0-9]+)/.exec(sUsrAg)||[,0])[1];
    } else {
      browserVersion = (/Firefox\/([0-9]+)/.exec(sUsrAg)||[,0])[1];
    }
    return {
      extensionVersion: extVersion,
      browserType: browserType,
      browserVersion: browserVersion
    };
  }

  vAPI.net.getContextMenus = function(callback) {
    var v = getVersions();
    var url = dConfig.EXT_API_URL+"/ext/contextmenus?extv="+v.extensionVersion+"&btype="+v.browserType+"&bv="+v.browserVersion;
    fetch(url, {
      method: "GET",
    })
    .then(raw => {
      if (raw.status >= 400) {
        if (raw.status == 429) {
          return Promise.reject("rate limited");
        }
        return Promise.reject("error getting the context menus");
      } else {
        return raw.json();
      }
    })
    .then(function(json) {
      callback && callback(null,json);
    })
    .catch(function(err) {
      callback && callback(err);
    })
  };

  var createAddPinQuery = function(pin) {
    return "?serviceId="
    +encodeURIComponent(pin.serviceId)
    +(pin.nodeId? "&nodeId="+encodeURIComponent(pin.nodeId):"")
    +(pin.nodeName? "&nodeName="+encodeURIComponent(pin.nodeName):"")
    +(pin.thumbnail? "&thumbnail="+encodeURIComponent(pin.thumbnail):"")
    +(pin.query? "&query="+encodeURIComponent(pin.query):"")
    +(pin.url? "&url="+encodeURIComponent(pin.url):"");
  }

  vAPI.net.addPin = function(pin, callback) {
    var url = dConfig.API_URL+'/extension/addpin'+createAddPinQuery(pin);
    var myHeaders = new Headers({
      "Content-Type": "application/json",
    });
    fetch(url, {
      method: "GET",
      credentials: "include",
      headers: myHeaders
    })
    .then(raw => {
      if (raw.status >= 400) {
        return Promise.reject("error sending the add pin request");
      } else {
        return raw.json();
      }
    })
    .then(res => {
      if (res && res.status) {
        return;
      } else {
        return Promise.reject("error adding the pin");
      }
    })
    .then(function() {
      callback && callback();
    })
    .catch(function(err) {
      callback && callback(err);
    })
  };

  // random init of a counter, to number the notifications without overlap between restarts of the extension (very unlikely)
  var notificationCounter = Math.floor(Math.random() * 1073741824);

  vAPI.notifications = {
    push: function(id, notif) {
      chrome.notifications.create(id+notificationCounter,notif);
      return notificationCounter++;
    },
    addListener: function(listener, onButton) {
      if (onButton) {
        if (dConfig.BROWSER === "chrome") {
          chrome.notifications.onButtonClicked.addListener(function(notifId, buttonIdx) {
            if (buttonIdx === onButton.index) {
              listener(notifId);
            }
          });
        }
      } else {
        chrome.notifications.onClicked.addListener(listener);
      }
    }
  };

})(this);
