'use strict';

var dClient = window.dClient = (function() {
  return {
    _elements: {
      iframe: null,
      spinnerDiv: null,
      spinnerStyle: null,
      documentBody: null,
      documentHead: null
    }
  };
})();
(function(self) {
  self.getElement = {
    iframe: function() {
      return self._elements.iframe?
        self._elements.iframe :
        self._elements.iframe = document.getElementById('joliframe');
    },
    spinnerDiv: function() {
      return self._elements.spinnerDiv?
        self._elements.spinnerDiv :
        self._elements.spinnerDiv = document.getElementById('waiting-spinner-desktop');
    },
    spinnerStyle: function() {
      return self._elements.spinnerStyle;
    },
    documentBody: function() {
      return self._elements.documentBody?
        self._elements.documentBody :
        self._elements.documentBody = document.getElementsByTagName('body')[0];
    },
    documentHead: function() {
      return self._elements.documentHead?
        self._elements.documentHead :
        self._elements.documentHead = document.getElementsByTagName('head')[0];
    },
  };
  self.setElement = {
    iframe: function(url) {
      var iframe = document.createElement('iframe');
      iframe.setAttribute('style', 'display: block; width: 100%; height: 100%; position: fixed; top: 0; left: 0; border: none; z-index: 2147483642;');
      iframe.setAttribute('id',    'joliframe');
      iframe.setAttribute('src',   url);
      self.getElement.documentBody().appendChild(iframe);
      return self._elements.iframe = iframe;
    },
    spinnerStyle: function() {
      var spinnerStyleURL = chrome.extension.getURL("spinnerstyle.css");
      var link = document.createElement('link');
      link.type = 'text/css';
      link.rel = 'stylesheet';
      link.href = spinnerStyleURL;
      self.getElement.documentHead().appendChild(link);
      return self._elements.spinnerStyle = link;
    },
    spinnerDiv: function() {
      self.setElement.spinnerStyle();
      var spinnerDiv = document.createElement('div');
      spinnerDiv.setAttribute('style', 'display: block; width: 50px; height: 50px; position: fixed; top: 50%; left: 50%; margin-top: -25px; margin-left: -25px; border: none; z-index: 2147483643; background-color: rgba(0, 0, 0, 0.3); border-radius: 8px 0px 8px 8px;');
      spinnerDiv.setAttribute('id', 'waiting-spinner-desktop');
      var internalSpinner = document.createElement('div');
      internalSpinner.className += "vjs-loading-spinner";
      spinnerDiv.appendChild(internalSpinner);
      var closeImgURL = chrome.extension.getURL("assets/images/button-x.png");
      var close_buttonDiv = document.createElement('div');
      close_buttonDiv.setAttribute('style', 'display: block; width: 20px; height: 20px;cursor: pointer; position: relative; margin-top: -10px; margin-left: 40px; border: none; z-index: 2147483645; background-image: url("'+closeImgURL+'"); background-repeat: no-repeat; background-position: center center; background-size: 20px 20px;');
      close_buttonDiv.setAttribute('id', 'close-spinner-desktop');
      close_buttonDiv.onclick = function() {
        self.content.removeAll();
      };
      spinnerDiv.appendChild(close_buttonDiv);
      self.getElement.documentBody().appendChild(spinnerDiv);
      return self._elements.spinnerDiv = spinnerDiv;
    }
  };
  self.clearElement = {
    iframe: function() {
      var iframe = self.getElement.iframe();
      iframe && iframe.parentNode.removeChild(iframe);
      self._elements.iframe = null;
    },
    spinnerDiv: function() {
      var spinnerDiv = self.getElement.spinnerDiv();
      spinnerDiv && spinnerDiv.parentNode.removeChild(spinnerDiv);
      self._elements.spinnerDiv = null;
    },
  }
  self.bodyStyle = {
    set: function() {
      self.getElement.documentBody()
      .setAttribute('style',
        (self.getElement.documentBody().getAttribute('style') || '') + ' overflow: hidden;'
      );
    },
    unset: function() {
      self.getElement.documentBody()
      .setAttribute('style',
        self.getElement.documentBody()
        .getAttribute('style').replace(' overflow: hidden;', '')
      );
    }
  };
  self.adBlockObserver = {
    _observer: null,
    set: function() {
      var iframe = self.getElement.iframe();
      if (iframe) {
        this._observer = new MutationObserver(function(mutations) {
          mutations.forEach(function(mutationRecord) {
            if (iframe.style.display !== "block") {
              console.error('iframe blocked by adblocker!');
              var adblockerNotice = document.createElement('div');
              adblockerNotice.setAttribute('id', "adblock-notice");
              adblockerNotice.innerHTML = "extension blocked by Adblocker!";
              spinnerDiv.appendChild(adblockerNotice);
            }
          });
        });
        this._observer.observe(iframe, { attributes : true, attributeFilter : ['style'] });
      }
    },
    unset: function() {
      this._observer && this._observer.disconnect();
      this._observer = null;
    }
  }

  self.content = {
    removeAll : function(){
      self.adBlockObserver.unset();
      self.clearElement.iframe();
      self.clearElement.spinnerDiv();
      self.bodyStyle.unset();
    },
    unspin: function() {
      self.adBlockObserver.unset();
      self.clearElement.spinnerDiv();
    },
    addFolderPicker: function(iframeUrl) {
      self.setElement.spinnerDiv();
      self.setElement.iframe(iframeUrl);
      self.adBlockObserver.set();
      self.bodyStyle.set();
    },
    addAddPin: function(iframeUrl) {
      self.setElement.spinnerDiv();
      self.setElement.iframe(iframeUrl);
      self.adBlockObserver.set();
      self.bodyStyle.set();
    }
  };


  self.containsFeed = function(doc) {

    // Find all the RSS link elements.
    var result = doc.evaluate(
        '//*[local-name()="rss" or local-name()="feed" or local-name()="RDF"]',
        doc, null, 0, null);

    if (!result) {
      return false;  // This is probably overly defensive, but whatever.
    }

    var node = result.iterateNext();

    if (!node) {
      return false;  // No RSS tags were found.
    }

    if (node.parentElement && node.parentElement.tagName != "BODY") {
      return false;
    }

    return true;
  };

  self.isFeed = function() {
    if (self.containsFeed(document)) {
      vAPI.sendMessage({feed: location.href});
      return true;
    }
    return false;
  };

  self.init = function() {
    vAPI.onMessage.addListener(function(request, sender, sendResponse) {
      var urlTmp;
      if (request.action === 'addFolderPicker') {
        urlTmp = dConfig.API_URL+'/ext/folderpicker?url=' + window.btoa(encodeURIComponent(request.item.url)) + '&encode=base64&name=' + encodeURIComponent(request.item.name) + '&type=' + encodeURIComponent(request.item.type) + '&edit=' + request.edit;
        if (request.item.exportType) {
          urlTmp+= '&exporttype=' + encodeURIComponent(request.item.exportType);
        }
        if (chrome.runtime.id) {
          urlTmp+= '&extid=' + encodeURIComponent(vAPI.extid);
        }
        if (dConfig.BROWSER === "firefox") {
          urlTmp+= '&browsertarget=firefox';
        }
        self.content.addFolderPicker(urlTmp);
      } else if (request.action === 'addAddPin') {
        urlTmp = dConfig.API_URL+'/ext/addpin?url=' + window.btoa(encodeURIComponent(request.url)) + '&encode=base64&menuitemid=' + encodeURIComponent(request.menuItemId);
        if (chrome.runtime.id) {
          urlTmp+= '&extid=' + encodeURIComponent(vAPI.extid);
        }
        if (dConfig.BROWSER === "firefox") {
          urlTmp+= '&browsertarget=firefox';
        }
        self.content.addAddPin(urlTmp);
      } else if (request.action === 'remove') {
        self.content.removeAll();
      } else if (request.action === 'unspin') {
        self.content.unspin();
      }
      sendResponse({status: "success"});
    });

    if (dConfig.BROWSER === "firefox") {
      vAPI.onPageMessage.addListener(function(message) {
        if (message.data &&
            (message.data.source == "dissident-folderpicker" || message.data.source == "dissident-addpinmodal")) {
          if (message.data.content) {
            var data = message.data.content;
            if (data.action) {
              if (data.action === "unspin") {
                self.content.unspin();
              } else if (data.action === 'remove') {
                self.content.removeAll();
              }
            } else if (data.upload || data.addpin) {
              vAPI.sendMessage(data);
            }
          }
        }
      });
    }
  }
  if (!self.isFeed()) {
    self.init();
  }
})(dClient);
