/* Copyright (c) 2009 The Chromium Authors. All rights reserved.
   Use of this source code is governed by a BSD-style license that can be
   found in the LICENSE file.
*/

/* Use only multi-line comments in this file, since during testing
   its contents will get read from disk and stuffed into the
   iframe .src tag, which is a process that doesn't preserve line
   breaks and makes single-line comment out the rest of the code.
*/

/* The maximum number of feed items to show in the preview. */
var maxFeedItems = 10;

/* The maximum number of characters to show in the feed item title. */
var maxTitleCount = 1024;
var viewSource = false;
var data;
var built = false;
window.addEventListener("message", function(e) {
  if (e.data.rssText) {
    data = e.data.rssText;
    parseAndBuildPreview()
  } else if (e.data.viewSource) {
    viewSource = e.data.viewSource === "source";
    parseAndBuildPreview()
  }

}, false);

function parseAndBuildPreview() {
  if (built) {
    document.body.innerHTML = '';
  }
  if (!data) {
    return;
  }
  if (viewSource) {
    var preElem = document.createElement("pre");
    preElem.lang="xml";
    preElem.innerText = data;
    document.body.appendChild(preElem);
    built = true;
  } else {
    var parser = new DOMParser();
    var doc = parser.parseFromString(data, "text/xml");

    if (doc) {
      built = true;
      buildPreview(doc);
    } else {
      /* Already handled in subscribe.html */
    }
  }
}


function formatDescription(span) {
    var images = span.getElementsByTagName('img');
    for (var i = 0; i < images.length; i++) {
      images[i].style.maxWidth = "300px";
      images[i].style.maxHeight = "300px";
    }
    return span;
}

function stripHtml(str) {
  return str.replace(/<.*?>/g, '');
}

function fancyDate(date) {
  var dateFormated = moment(date).calendar(null, {
    sameDay: "[Today]",
    nextDay: "[Tomorrow]",
    nextWeek: "dddd",
    lastDay: "[Yesterday]",
    lastWeek: "dddd",
    sameElse: "LL",
  });
  dateFormated = dateFormated.replace(", "+moment().year().toString(), "");
  return dateFormated;
};

function buildPreview(doc) {
  /* Start building the part we render inside an IFRAME. We use a table to
     ensure that items are separated vertically from each other. */
  var table = document.createElement("table");
  table.id = "main-rss-table";
  table.setAttribute("cellspacing","0");
  var tbody = document.createElement("tbody");
  table.appendChild(tbody);

  /* Now parse the rest. Some use <entry> for each feed item, others use
     <channel><item>. */
  var entries = doc.getElementsByTagName('entry');
  if (entries.length == 0)
    entries = doc.getElementsByTagName('item');

  for (i = 0; i < entries.length && i < maxFeedItems; ++i) {
    item = entries.item(i);

    /* Grab the title for the feed item. */
    var itemTitle = item.getElementsByTagName('title')[0];
    if (itemTitle)
      itemTitle = itemTitle.textContent;
    else
      itemTitle = "Unknown title";

    /* Ensure max length for title. */
    if (itemTitle.length > maxTitleCount)
      itemTitle = itemTitle.substring(0, maxTitleCount) + "...";

    /* Grab the date */
    var itemDate = null;
    var dateElements = item.querySelectorAll("pubdate,pubDate,lastbuilddate,lastBuildDate,published,modified,updated,dc\\:date");
    if (dateElements.length > 0) {
      var date,tagname;
      for (var j = 0; j < dateElements.length; j++) {
        date = new Date(dateElements[j].textContent);
        if (date instanceof Date && !isNaN(date)) {
          tagname = dateElements[j].tagName;
          if (itemDate === null || tagname.toLowerCase() == 'lastbuilddate' || tagname == 'modified' || tagname == 'updated') {
            itemDate = date;
          }
        }
      }
    }
    /* Grab the description.
       TODO(aa): Do we need to check for type=html here? */
    var itemDesc = item.getElementsByTagName('description')[0];
    if (!itemDesc || !itemDesc.textContent.replace(/\s/g,""))
      itemDesc = item.getElementsByTagName('summary')[0];
    if (!itemDesc || !itemDesc.textContent.replace(/\s/g,""))
      itemDesc = item.getElementsByTagName('content')[0];
    if (!itemDesc || !itemDesc.textContent.replace(/\s/g,""))
      itemDesc = item.getElementsByTagName('content:encoded')[0];

    if (itemDesc)
      itemDesc = itemDesc.textContent;
    else
      itemDesc = "";

    var image = null;
    var imageElem = null;
    var tdImg = null;
    var feedlyImgRegexp = /\<img.+?(?:class\=(?:\"|\').*webfeedsFeaturedVisual.*(?:\"|\'))?.*?src\=(?:\"|\')(.+?)(?:\"|\')(?:.+?)(?:class\=(?:\"|\').*?webfeedsFeaturedVisual.*?(?:\"|\'))?\>/;
    var regexp = /\<img.+?src\=(?:\"|\')(.+?)(?:\"|\')(?:.+?)\>/;
    var extractPrio = null;
    var extract = null;
    var itemDescForImg = item.getElementsByTagName('description')[0] && item.getElementsByTagName('description')[0].textContent;
    extract = itemDescForImg && regexp.exec(itemDescForImg);
    if (!extract || !extract[1]) {
      itemDescForImg = item.getElementsByTagName('summary')[0] && item.getElementsByTagName('summary')[0].textContent;
      extract = itemDescForImg && regexp.exec(itemDescForImg);
    }
    if (!extract || !extract[1]) {
      itemDescForImg = item.getElementsByTagName('content')[0] && item.getElementsByTagName('content')[0].textContent;
      extract = itemDescForImg && regexp.exec(itemDescForImg);
    }
    if (!extract || !extract[1]) {
      itemDescForImg = item.getElementsByTagName('content:encoded')[0] && item.getElementsByTagName('content:encoded')[0].textContent;
      extract = itemDescForImg && regexp.exec(itemDescForImg);
    }
    extractPrio = feedlyImgRegexp.exec(itemDescForImg);
    if (extract && extract[1]) {
      image = (extractPrio && extractPrio[1]) || extract[1];
    }
    if (!image) {
      image = item.getElementsByTagName('image')[0] || item.getElementsByTagName('logo')[0];
      if (image) {
        if (image.getElementsByTagName('url')[0]) {
          image = image.getElementsByTagName('url')[0].textContent;
        } else {
          image = image.textContent;
        }
      }
    }
    if (!image) {
      image = item.getElementsByTagName('itunes:image')[0];
      image = image && image.getAttribute("href");
    }

    if (!image) {
      var enclosures = item.getElementsByTagName('enclosure');
      if (enclosures.length > 0) {
        var typeAttribute;
        for (var j = 0; j < enclosures.length; j++) {
          typeAttribute = enclosures[j].getAttribute("type");
          if (typeAttribute && typeAttribute.indexOf("image") === 0) {
            image = enclosures[j].getAttribute("url");
            break;
          }
        }
      }
    }
    if (!image) {
      image = item.getElementsByTagName('media:thumbnail')[0];
      image = image && image.getAttribute("url");
    }
    if (!image) {
      image = item.getElementsByTagName('g:image_link')[0];
      image = image && image.textContent;
    }
    if (!image) {
      var links = item.getElementsByTagName("link");
      if (links.length > 0) {
        var rel,type;
        for (var j = 0; j < links.length; j++) {
          rel = links[j].getAttribute("rel");
          if (rel === 'alternate' || rel === 'self' || rel === 'replies' || rel === 'enclosure') {
            type = links[j].getAttribute("type");
            if (type && type.indexOf("image") === 0) {
              image = links[j].getAttribute("href");
              break;
            }
          }
        }
      }
    }
    if (!image) {
      image = vAPI.getURL("/assets/images/rss_feed/placeholder-article.png")
    }


    imageElem = document.createElement("div");
    imageElem.classList.add("rss_main_img")
    imageElem.style.backgroundImage = "url('"+image+"')";
    imageWrap = document.createElement("div");
    imageWrap.classList.add("rss_main_img_wrapper");
    tdImg = document.createElement("td");
    tdImg.classList.add("rss_main_img_cell");
    imageWrap.appendChild(imageElem)
    tdImg.appendChild(imageWrap);

    /* Grab the link URL. */
    var itemLink = item.getElementsByTagName('link');
    var link = "";
    if (itemLink.length > 0) {
      link = itemLink[0].childNodes[0];
      if (link)
        link = itemLink[0].childNodes[0].nodeValue;
      else
        link = itemLink[0].getAttribute('href');
    }



    var tr = document.createElement("tr");
    var td = document.createElement("td");
    td.classList.add('text_cell');
    tr.classList.add("rss_row");
    if (i%2 === 0) {
      tdImg.classList.add('whitebg');
      td.classList.add('whitebg');
    } else {
      tdImg.classList.add('greybg');
      td.classList.add('greybg');
    }
    /* If we found a link we'll create an anchor element,
    otherwise just use a bold headline for the title. */
    var anchor = (link != "") ? document.createElement("a") :
                                document.createElement("strong");
    anchor.id = "anchor_" + String(i);
    if (link != "")
      anchor.href = link;
    anchor.innerHTML = itemTitle;
    anchor.target = "_top";
    anchor.className = "item_title";
    var dateDiv;
    if (itemDate) {
      dateDiv = document.createElement("div");
      dateDiv.id = "date_" + String(i);
      dateDiv.className = "item_date";
      dateDiv.innerHTML = fancyDate(itemDate);
    }
    var span = document.createElement("div");
    span.id = "desc_" + String(i);
    span.className = "item_desc";
    span.innerHTML = stripHtml(itemDesc);
    td.appendChild(anchor);
    // dateDiv && td.appendChild(document.createElement("br"));
    dateDiv && td.appendChild(dateDiv);
    // td.appendChild(document.createElement("br"));
    td.appendChild(span);

    tr.appendChild(tdImg);
    tr.appendChild(td);
    tbody.appendChild(tr);
  }

  table.appendChild(tbody);
  document.body.appendChild(table);
}
