// Copyright (c) 2012 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Grab the querystring, removing question mark at the front and splitting on
// the ampersand.
var queryString = location.search.substring(1).split("&");

// The feed URL is the first component and always present.
var feedUrl = decodeURIComponent(queryString[0]);

// The XMLHttpRequest object that tries to load and parse the feed, and (if
// testing) also the style sheet and the frame js.
var req;

// What to show when we cannot parse the feed name.
var unknownName = "RSS Feed";

var viewSource = false;
var iframe;
/**
* The main function. Sets up the selection list for possible readers and
* fetches the data.
*/
function main() {

  // Now fetch the data.
  req = new XMLHttpRequest();

  req.onload = handleResponse;
  req.onerror = handleError;
  // Not everyone sets the mime type correctly, which causes handleResponse
  // to fail to XML parse the response text from the server. By forcing
  // it to text/xml we avoid this.
  req.overrideMimeType('text/xml');
  req.open("GET", feedUrl, true);
  req.send(null);

}

// Sets the title for the feed.
function setFeedTitle(title) {
  var titleTag = document.getElementById('title');
  titleTag.textContent = title;
}

// Handles errors during the XMLHttpRequest.
function handleError() {
  handleFeedParsingFailed("Error fetching feed.");
}

// Handles feed parsing errors.
function handleFeedParsingFailed(error) {
  setFeedTitle(unknownName);

  // The tests always expect an IFRAME, so add one showing the error.
  var html = "<body><span id=\"error\" class=\"item_desc\">" + error +
               "</span></body>";

  var error_frame = createFrame('error', html);
  var itemsTag = document.getElementById('items');
  itemsTag.appendChild(error_frame);
}

function createFrame(frame_id) {
  var csp = '<meta http-equiv="content-security-policy" ' +
            'value="script-src \'self\'; object-src \'none\'">';
  var frame = document.createElement('iframe');
  frame.id = frame_id;
  frame.src = vAPI.getURL("rss_feed/rss_inject.html")
  frame.scrolling = "auto";
  frame.frameBorder = "0";
  frame.marginWidth = "0";
  return frame;
}

function embedAsIframe(rssText) {
  var itemsTag = document.getElementById('items');

  // TODO(aa): Add base URL tag
  iframe = createFrame('rss');
  itemsTag.appendChild(iframe);

  iframe.onload = function() {
    iframe.contentWindow.postMessage({rssText:rssText}, "*");
  }
}

// Handles parsing the feed data we got back from XMLHttpRequest.
function handleResponse() {
  // Uncomment these three lines to see what the feed data looks like.
  // var itemsTag = document.getElementById('items');
  // itemsTag.textContent = req.responseText;
  // return;

  var doc = req.responseXML;
  if (!doc) {
    // If the XMLHttpRequest object fails to parse the feed we make an attempt
    // ourselves, because sometimes feeds have html/script code appended below a
    // valid feed, which makes the feed invalid as a whole even though it is
    // still parsable.
    var domParser = new DOMParser();
    doc = domParser.parseFromString(req.responseText, "text/xml");
    if (!doc) {
      handleFeedParsingFailed("Not a valid feed.");
      return;
    }
  }

  // We must find at least one 'entry' or 'item' element before proceeding.
  var entries = doc.getElementsByTagName('entry');
  if (entries.length == 0)
    entries = doc.getElementsByTagName('item');
  if (entries.length == 0) {
    handleFeedParsingFailed("This feed contains no entries.")
    return;
  }

  // Figure out what the title of the whole feed is.
  var title = doc.getElementsByTagName('title')[0];
  if (title)
    setFeedTitle(title.textContent);
  else
    setFeedTitle(unknownName);

  // Add an IFRAME with the html contents.
  embedAsIframe(req.responseText);
}

function addpin() {
  vAPI.sendMessage({addpinfeed:feedUrl});
}

function viewSourceToggle() {
  viewSource = !viewSource;
  iframe.contentWindow.postMessage({viewSource: viewSource?"source":"feed"}, "*");
}

document.addEventListener('DOMContentLoaded', function () {
  var button = document.getElementById('rss_addpin_button');
  button.addEventListener('click', addpin);
  var viewSourceButton = document.getElementById("corner_action");
  viewSourceButton.addEventListener('click', viewSourceToggle);
  main();
});
